package gregapi.item;

import gregapi.code.TagData;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;

/**
 * @author Gregorius Techneticies
 * 
 * Interface for Items with some kind of Charge.
 */
public interface IItemEnergy {
	// for tampering with the charge of an Object.
	// 
	// The World and Coordinate Parameters are there in case something should happen when tampering with the Item.
	//   This could be used for remote re-chargable Items.
	// The Inventory Parameters are for the Inventory the Item is inside.
	//   If it is inside a Player, then please always add the Player Inventory.
	//   If this is inside a TileEntity, then you are not required to add the IInventory, even if it has an Inventory.
	
	/**
	 * You do not have to check for this Function, this is only for things like Slot insertion Conditions and similar.
	 * @param aEnergyType The Type of Energy
	 * @param aEmitting if it is asked emit this Energy Type, otherwise it is asked to accept this Energy Type.
	 * @return if this Item has anything to do with this Type of Energy, depending on insert or extract request. The returning Value must be constant for this Item.
	 */
	public boolean isEnergyType(TagData aEnergyType, ItemStack aStack, boolean aEmitting);
	
	/**
	 * Charges an Energy Storage. The Coordinates represent the position of wherever the Item is supposed to be.
	 * @param aEnergyType The Type of Energy
	 * @param aWorld if the Coordinates passed are valid then this must be != null. if this is null, then the Position is unknown.
	 * @param aInventory the Inventory the Item is in. May be null.
	 * @param aDoInject if this is supposed to increase the internal Energy. true = Yes, this is a normal Operation. false = No, this is just a simulation.
	 * @return amount of used aAmount.
	 */
	public long doEnergyInjection(TagData aEnergyType, ItemStack aStack, long aSize, long aAmount, IInventory aInventory, World aWorld, int aX, int aY, int aZ, boolean aDoInject);
	
	/**
	 * Decharges an Energy Storage. The Coordinates represent the position of wherever the Item is supposed to be.
	 * @param aEnergyType The Type of Energy
	 * @param aWorld if the Coordinates passed are valid then this must be != null. if this is null, then the Position is unknown.
	 * @param aDoExtract if this is supposed to decrease the internal Energy. true = Yes, this is a normal Operation. false = No, this is just a simulation.
	 * @return amount of taken aAmount.
	 */
	public long doEnergyExtraction(TagData aEnergyType, ItemStack aStack, long aSize, long aAmount, IInventory aInventory, World aWorld, int aX, int aY, int aZ, boolean aDoExtract);
	
	/**
	 * "canUse" and "use" together in one Function by using a Simulate Parameter.
	 * @param aEnergyType The Type of Energy
	 * @param aWorld if the Coordinates passed are valid then this must be != null. if this is null, then the Position is unknown.
	 * @param aDoUse if this is supposed to decrease the internal Energy. true = Yes, this is a normal Operation. false = No, this is just a simulation.
	 * @return true if this can use that much Energy.
	 */
	public boolean useEnergy(TagData aEnergyType, ItemStack aStack, long aSize, long aAmount, IInventory aInventory, World aWorld, int aX, int aY, int aZ, boolean aDoUse);
	
	/**
	 * @param aEnergyType The Type of Energy.
	 * @return the ItemStack you used as a Parameter. This is for convenience.
	 */
	public ItemStack setEnergyStored(TagData aEnergyType, ItemStack aStack, long aAmount);
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return amount of Energy stored.
	 */
	public long getEnergyStored(TagData aEnergyType, ItemStack aStack);
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return amount of Energy that can be stored. 0 if not accepting this Energy.
	 */
	public long getEnergyCapacity(TagData aEnergyType, ItemStack aStack);
	
	
	// Parameters for the minimum Packet Sizes in order for things to actually work.
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The minimum Energy Packet Size for the INPUT of this Object.
	 */
	public long getEnergySizeInputMin(TagData aEnergyType, ItemStack aStack);
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The minimum Energy Packet Size for the OUTPUT of this Object.
	 */
	public long getEnergySizeOutputMin(TagData aEnergyType, ItemStack aStack);
	
	
	// Parameters for the recommended Packet Sizes.
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The recommended Energy Packet Size for the INPUT of this Object.
	 */
	public long getEnergySizeInputRecommended(TagData aEnergyType, ItemStack aStack);
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The recommended Energy Packet Size for the OUTPUT of this Object.
	 */
	public long getEnergySizeOutputRecommended(TagData aEnergyType, ItemStack aStack);
	
	
	// Parameters for the maximum Packet Sizes before bad shit happens.
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The maximum Energy Packet Size for the INPUT of this Object.
	 */
	public long getEnergySizeInputMax(TagData aEnergyType, ItemStack aStack);
	
	/**
	 * @param aEnergyType The Type of Energy
	 * @return The maximum Energy Packet Size for the OUTPUT of this Object.
	 */
	public long getEnergySizeOutputMax(TagData aEnergyType, ItemStack aStack);
}